from . import validation
from .mdread import markdown_from_file, process_template
from .outputformatter import OutputFormatter
from .latexformatter import LatexFormatter
from .htmlformatter import HTMLFormatter
from .csvformatter import CSVFormatter
from .riskassessment import RiskAssessment
from .vulnerability import VulnerabilityList
import os
import jinja2
import datetime
import logging
import traceback

log = logging.getLogger(__name__)


class Template:
    """ A Report Ranger template for building a report from a collection of markdown. """

    def __init__(self, templatefile='', templatemapper={}, defaulttemplate=''):
        self.templatefile = templatefile
        self.templatemapper = templatemapper
        self.defaulttemplate = defaulttemplate

    def _process_included_header(self, headers, includedheaders):
        """ Process the included headers using an overlay routine:
        - Don't overwrite values
        - Add new values to dicts if they're not there
        - Append to lists
        - reset values using the "reset" header
        """
        if isinstance(includedheaders, dict):
            if not isinstance(headers, dict):
                return headers

            if 'reset' in headers:
                log.info("Resetting {} in included template headers".format(
                    headers['reset']))
                if isinstance(headers['reset'], list):
                    for r in headers['reset']:
                        if isinstance(r, str):
                            if r in includedheaders:
                                del includedheaders[r]
                        else:
                            log.warning(
                                "Reset variable {} not a string, skipping".format(r))
                else:
                    log.warning(
                        "Reset list {} not a list, skipping".format(headers['reset']))

            for header in includedheaders:
                if header in headers:
                    header = self._process_included_header(
                        headers[header], includedheaders[header])
                else:
                    headers[header] = includedheaders[header]
            return headers
        elif isinstance(includedheaders, list):
            if not isinstance(headers, list):
                return headers
            headers.extend(includedheaders)
            return headers
        else:
            return headers

    # This includes the headers from another file. This is convenient for stock text, included risk assessments, etc
    def _include_markdown(self, templateheaders, templatemarkdown, templatemapper, directory=None):
        if 'include_markdown' in templateheaders:
            if not isinstance(templateheaders['include_markdown'], str):
                log.warning(
                    "include_markdown file in template is not a string, skipping")
                return templatemarkdown

            included = templateheaders['include_markdown']

            log.info("Template including markdown from {}".format(included))
            # See if it's in the template mapper
            if included in templatemapper:
                log.info("Template markdown {} in templatemapper, changing it to {}".format(
                    included, templatemapper[included]))
                included = templatemapper[included]
            elif directory:
                included = os.path.join(directory, included)

            fn, ext = os.path.splitext(included)
            if ext != ".md" and ext != ".rr":
                log.warning(
                    "File {} included in include_markdown in the template does not end in .md or .rr, skipping".format(included))
                return templatemarkdown
            if not os.path.isfile(included):
                log.warning(
                    "File {} included in include_markdown in the template does not exist, skipping".format(included))
                return templatemarkdown

            _, ntmarkdown = markdown_from_file(included)
            return ntmarkdown
        return templatemarkdown

    # This includes the headers from another file. This is convenient for stock text, included risk assessments, etc
    def _include_headers(self, headers, templatemapper, directory=''):
        if 'include' in headers:
            if isinstance(headers['include'], str):
                headers['include'] = [headers['include']]
            for included in headers['include']:
                log.info("Template including file {}".format(included))
                # See if it's in the template mapper
                if included in templatemapper:
                    log.info("Including template {} at {}".format(
                        included, templatemapper[included]))
                    included = templatemapper[included]
                elif directory:
                    included = os.path.join(directory, included)

                fn, ext = os.path.splitext(included)
                if ext != ".md" and ext != ".rr":
                    log.warning(
                        "File {} included in the template does not end in .md or .rr, skipping".format(included))
                    return headers
                if not os.path.isfile(included):
                    log.warning(
                        "File {} included in the template does not exist, skipping".format(included))
                    return headers

                includedheaders, _ = markdown_from_file(included)
                headers = self._process_included_header(
                    headers, includedheaders)

        return headers

    # If the "change_template" header is there the report can set its own template variables
    def _change_template_headers(self, templateheaders, reportheaders):
        if 'change_template' in reportheaders:
            log.info("change_template header detected")
            if not isinstance(reportheaders['change_template'], dict):
                log.warning(
                    "change_template front matter variable not a dictionary")
                return templateheaders

            # Notice this is back to front, the template itself is included into the report change_template variable
            templateheaders = self._process_included_header(
                reportheaders['change_template'], templateheaders)

        return templateheaders

    def init_template(self, templatefile, reportheaders):
        """ Initialise the template.

        The template should be in the form (variables, markdown).
        """
        log.info("Initialising template {}".format(templatefile))

        if templatefile in self.templatemapper:
            templatefile = self.templatemapper[templatefile]

        self.templatepath = os.path.abspath(
            os.path.join(os.path.curdir, templatefile))

        self.templateheaders, self.templatemarkdown = markdown_from_file(
            templatefile, False)

        # Process include_markdown

        # Get any change template headers in the reportbody and insert it into the template
        self.templateheaders = self._change_template_headers(
            self.templateheaders, reportheaders)

        # Process templatedir
        if 'templatedir' in self.templateheaders:
            templatedirfile = self.templateheaders['templatedir']
            if not isinstance(templatedirfile, str):
                log.warning(
                    "templatedir front matter variable not a string, skipping")
                self.templatedir = os.path.dirname(os.path.abspath(
                    os.path.join(os.path.curdir, templatefile)))
            else:
                if templatedirfile in self.templatemapper:
                    templatedirfile = self.templatemapper
                self.templatedir = os.path.dirname(os.path.abspath(
                    os.path.join(os.path.curdir, templatedirfile)))
        else:
            self.templatedir = os.path.dirname(os.path.abspath(
                os.path.join(os.path.curdir, templatefile)))

        # Get any includes
        self.templateheaders = self._include_headers(
            self.templateheaders, self.templatemapper, self.templatedir)

        self.templatemarkdown = self._include_markdown(
            self.templateheaders, self.templatemarkdown, self.templatemapper, self.templatedir)

        self.templateheaders['templatedir'] = self.templatedir

        self.latex_template = os.path.join(
            self.templatedir, self.templateheaders['latex']['template'])

        if 'html' in self.templateheaders:
            self.html_template = os.path.join(
                self.templatedir, self.templateheaders['html']['template'])
        else:
            self.html_template = ''

        # Initialise the risk assessment
        self.riskassessment = RiskAssessment(
            self.templateheaders['riskassessment'])
        if 'style_text' in self.templateheaders['riskassessment']:
            self.templateheaders['defaults']['ra_style_text'] = self.templateheaders['riskassessment']['style_text']

    def process_file(self, mdfile, target, docformat, output=None):
        """ Process a file with the template. Returns the processed markdown for the file.

        The target should be one of (latex, docx, pdf)
        """
        parentdir = os.path.dirname(os.path.join(os.path.curdir, mdfile))

        # Get the report
        reportheaders, reportbody = markdown_from_file(mdfile)

        # Figure out where to get the template
        if(self.templatefile != ''):  # Preference 1: If the user has directly asked for a template, use that
            # See if it's in the template mapper
            if self.templatefile in self.templatemapper:
                self.templatefile = self.templatemapper[self.templatefile]
                log.info(
                    "Supplied template found in template mapper: {}".format(self.templatefile))

            if not os.path.isfile(self.templatefile):
                log.warning(
                    "Template file does not exist, reverting to default")
                templatefile = self.defaulttemplate
            else:
                log.info("Template file provided: {}".format(
                    self.templatefile))
                templatefile = self.templatefile
        elif 'template' in reportheaders:  # Preference 2: The template is specified in the report file
            if reportheaders['template'] in self.templatemapper:
                templatefile = self.templatemapper[reportheaders['template']]
                log.info(
                    "Template file found in template mapper: {}".format(templatefile))
            else:
                log.warning("Template {} specified in report file, but it's not found in the templatemapper in config.".format(
                    reportheaders['template']))
                log.warning("Reverting to default template.")
                templatefile = self.defaulttemplate
        else:  # Preference 3: Just use the default
            templatefile = self.defaulttemplate
            log.info(
                "Default template used: {}".format(templatefile))

        self.init_template(templatefile, reportheaders)

        output_template = ''

        # Set up the appropriate output formatter
        if target == 'latex':
            of = LatexFormatter(target, self.templatedir,
                                self.templateheaders, headers=reportheaders)
            output_template = self.latex_template
        elif target == 'html':
            of = HTMLFormatter(target, self.templatedir,
                               self.templateheaders, headers=reportheaders)
            output_template = self.html_template
        elif target == 'csv':
            of = CSVFormatter(target, self.templatedir,
                              self.templateheaders, headers=reportheaders)
        else:  # Markdown formatter as default
            of = OutputFormatter(target, self.templatedir,
                                 self.templateheaders, headers=reportheaders)

        # Set up environment variables
        of.env.set_static('ra', self.riskassessment)
        of.env.set_static('templatedir', self.templatedir)

        # The title defaults to the filename without the extension
        if of.env.get('title') == None:
            log.warn("There's no title set in the front matter, the default title is the filename of {}".format(
                os.path.basename(mdfile)))
            of.env.set_variable('title', os.path.basename(mdfile))

        # Convert date to the date object
        if of.env.get('date') == None:
            # The date defaults to the last change, otherwise today
            if of.env.get('changes') != None:
                of.env.set_variable('date', of.env.get('changes')[-1][1])
                log.info("As there's no date in front matter, date of the latest change is used: {}".format(
                    of.env.get('date')))
            else:
                of.env.set_variable('date', datetime.date.today())
                log.info("As there's no date in front matter or changes, date has been set to today: {}".format(
                    of.env.get('date')))

        if of.env.get('client') == None:
            log.warning(
                "Client name is not in the report front matter. Default of [client] will be used.")
            of.env.set_variable('client', '[client]')

        if of.env.get('version') == None:
            if not of.env.get('changes') == None:
                of.env.set_variable('version', of.env.get('changes')[-1][0])
            else:
                of.env.set_variable('version', '1.0')
                log.warning(
                    "Version is not in the report front matter. Default of 1.0 will be used.")

        # Make sure that the risk assessment methodology is processed as RR markdown
        self.riskassessment.methodology_markdown = process_template(
            {}, self.riskassessment.methodology, env=of.env, name="Risk assessment methodology")

        vulnerability_validation = {}

        if 'validation' in self.templateheaders:
            if 'report' in self.templateheaders['validation']:
                validation.validate_headers(
                    self.templateheaders['validation']['report'], reportheaders, validation.default_report_validation)
            if 'vulnerability' in self.templateheaders['validation']:
                vulnerability_validation = self.templateheaders['validation']['vulnerability']

        # Get the vulnerabilities
        if of.env.get('vulndir'):
            # Resolve the vulnerability directory
            vulndir = os.path.join(os.path.dirname(
                mdfile), of.env.get('vulndir'))
            log.info("Vulnerability directory included: {}".format(vulndir))

            vulnerabilitylist = VulnerabilityList()
            try:
                vulnerabilitylist.add_from_dir(
                    vulndir, of, self.riskassessment, vulnerability_validation=vulnerability_validation)

                updated_date = of.env.get('updated_date')

                if updated_date:
                    vulnerabilitylist = vulnerabilitylist.updated(updated_date)

                of.env.set_static('vulnerabilities', vulnerabilitylist)
                of.env.set_static('vulns', vulnerabilitylist)

                # Generate vulnerability markdown
                vulnerabilitylist.generate_markdown(of.env)
            except Exception as e:
                log.warn("Error loading vulnerabilities: {}".format(e.args))
                log.warn(traceback.format_exc())
                log.warn(
                    "Error loading vulnerabilities, setting vulnerabilities as None")
                vulnerabilitylist = None
                of.env.set_static('vulnerabilities', None)
                of.env.set_static('vulns', None)

        # Render reportbody
        try:
            j2template = jinja2.Template(reportbody)
            rbrendered = j2template.render(of.env.get_env())
            of.env.set_static('reportbody', rbrendered)
        except jinja2.exceptions.TemplateSyntaxError as error:
            log.error("Jinja2 error: {} at lineno {} in reportbody for file {}".format(
                error.message, error.lineno, error.filename))
            of.env.set_static('reportbody', "")
        except Exception as error:
            log.error("Exception found in reportbody: {}".format(error.args))
            traceback.print_exc()
            log.error("Removing reportbody text")
            of.env.set_static('reportbody', "")

        return of.output(self.templatemarkdown, output_template, docformat, output, {})



from .errors import InputError
import os
from pathlib import Path
import re
from jinja2 import Template
import jinja2
import yaml
import json
from .imports import import_csv, import_xlsx
import logging
from . import section

log = logging.getLogger(__name__)

yamlmatch = re.compile(r'^[\.\=\-]{3}$')
jsonmatch = re.compile(r'^\;\;\;$')


def process_included_header(headers, includedheaders):
    """ Process the included headers using an overlay routine:
    - Don't overwrite values
    - Add new values if they're not there
    - Append to lists
    """
    if isinstance(includedheaders, dict):
        if not isinstance(headers, dict):
            return headers
        for header in includedheaders:
            if header in headers:
                header = process_included_header(
                    headers[header], includedheaders[header])
            else:
                headers[header] = includedheaders[header]
        return headers
    elif isinstance(includedheaders, list):
        if not isinstance(headers, dict):
            return headers
        headers.extend(includedheaders)
        return headers
    else:
        return headers


def process_template(headers, markdown, env=None, name='', filename=''):
    try:
        template = Template(markdown)  # Start jinja2
        env.push(headers)  # Add our new headers in
        output = template.render(env.get_env())
        env.pop()  # Take away the headers

    except jinja2.exceptions.TemplateSyntaxError as error:
        log.error(
            "Error in processing {} at {}".format(name, filename))
        log.error("Message: {}. Line number: {}.".format(
            error.message, error.lineno))
        log.error("Affected lines:")
        mdlines = markdown.splitlines()
        for i in range(12):
            el = error.lineno - 7 + i
            if el < 0:
                continue
            if el >= len(mdlines):
                continue
            log.error("{}: {}".format(el, mdlines[el]))
        raise Exception("Error reading {}: {} at lineno {} ".format(name,
                                                                    error.message, error.lineno))

    return output


def _process_includes(headers, includemapper):
    if isinstance(headers['include'], str):
        headers['include'] = [headers['include']]
    for included in headers['include']:
        # Process includemapper
        if included in includemapper:
            included = includemapper[included]
        filepath = Path(included)
        cwdpath = Path(os.path.curdir)
        if cwdpath not in filepath.parents:
            log.warn(
                "include path {} doesn't seem to be in cwd".format(filepath))
            continue
        log.info("Including file {}".format(filepath))
        includedheaders, _ = markdown_from_file(included)
        headers = process_included_header(
            headers, includedheaders)
    return headers


def _process_imports(headers):
    if not isinstance(headers['import'], dict):
        log.warn("import front matter variable not a dict, skipping")
    else:
        # Begin CSV imports
        if 'sections' in headers['import']:
            sectionlist = headers['import']['sections']
            if not isinstance(sectionlist, list):
                log.warn(
                    "Section import front matter variable not a list, skipping")
            for sectionimport in sectionlist:
                if not isinstance(sectionimport, dict):
                    log.warn(
                        "Section import item in front matter not a dict, skipping")
                    continue
                if 'directory' not in sectionimport or not isinstance(sectionimport['directory'], str):
                    log.warn(
                        "Section import item does not have a directory, skipping")
                    continue
                if 'variable' not in sectionimport or not isinstance(sectionimport['variable'], str):
                    log.warn(
                        "Section import item does not have variable name, skipping")
                    continue
                if sectionimport['variable'] in headers:
                    log.warn(
                        "Section import variable name already in front matter, skipping")
                    continue
                if 'ordinal' in sectionimport:
                    ordinal = sectionimport['ordinal']
                else:
                    ordinal = '1'

                # Build the section
                newsection = section.Section(
                    sectionimport['directory'], ordinal)

                headers[sectionimport['variable']] = newsection

        # Begin CSV imports
        if 'csv' in headers['import']:
            csvlist = headers['import']['csv']
            if not isinstance(csvlist, list):
                log.warn(
                    "csv import front matter variable not a list, skipping")
            for csvimport in csvlist:
                if not isinstance(csvimport, dict):
                    log.warn(
                        "csv import item in front matter not a dict, skipping")
                    continue
                if 'file' not in csvimport or not isinstance(csvimport['file'], str):
                    log.warn(
                        "csv import item does not have file location, skipping")
                    continue
                if 'variable' not in csvimport or not isinstance(csvimport['variable'], str):
                    log.warn(
                        "csv import item does not have variable name, skipping")
                    continue
                if csvimport['variable'] in headers:
                    log.warn(
                        "csv import variable name already in front matter, skipping")
                    continue
                if 'as_dict_list' in csvimport and not isinstance(csvimport['as_dict_list'], bool):
                    log.warning(
                        'csv import defines "as_dict_list", but it is not a boolean')
                    continue
                if 'index_col' in csvimport and not isinstance(csvimport['index_col'], int):
                    log.warning(
                        'csv import defines "index_col", but it is not an integer')
                    continue

                headers[csvimport['variable']] = import_csv(csvimport['file'], as_dict_list=csvimport.get(
                    'as_dict_list'), index_col=csvimport.get('index_col'))
        # Begin xlsx imports
        if 'xlsx' in headers['import']:
            xlsxlist = headers['import']['xlsx']
            if not isinstance(xlsxlist, list):
                log.warn(
                    "xlsx import front matter variable not a list, skipping")
            for xlsximport in xlsxlist:
                if not isinstance(xlsximport, dict):
                    log.warn(
                        "xlsx import item in front matter not a dict, skipping")
                    continue
                if 'file' not in xlsximport or not isinstance(xlsximport['file'], str):
                    log.warn(
                        "xlsx import item does not have file location, skipping")
                    continue
                if 'variable' not in xlsximport or not isinstance(xlsximport['variable'], str):
                    log.warn(
                        "xlsx import item does not have variable name, skipping")
                    continue
                if xlsximport['variable'] in headers:
                    log.warn(
                        "xlsx import variable name already in front matter, skipping")
                    continue
                if 'as_dict_list' in xlsximport and not isinstance(xlsximport['as_dict_list'], bool):
                    log.warning(
                        'xlsx import defines "as_dict_list", but it is not a boolean')
                    continue
                if 'index_col' in xlsximport and not isinstance(xlsximport['index_col'], int):
                    log.warning(
                        'xlsx import defines "index_col", but it is not an integer')
                    continue

                headers[xlsximport['variable']] = import_xlsx(
                    xlsximport['file'],
                    xlsximport.get('worksheet'),
                    xlsximport.get('min_row'),
                    xlsximport.get('max_row'),
                    xlsximport.get('min_col'),
                    xlsximport.get('max_col'),
                    as_dict_list=xlsximport.get('as_dict_list'),
                    index_col=xlsximport.get('index_col')
                )

    return headers


def markdown_from_file(file_loc, env=None, process_includes=True, includemapper={}, process_imports=True):
    """ Read the markdown from a file.

    Arguments:
    - file_loc: The location of the file to read

    Returns (headers, markdown) where headers is a dict and markdown is a long string """

    log.debug("Reading markdown from file {}".format(file_loc))

    if not os.path.exists(file_loc):
        log.warn("Trying to read file {} but it doesn't exist".format(file_loc))
        return dict(), ""

    with open(file_loc, 'r') as vf:
        # If the first line starts with "---" that's our headers We have to make sure that it starts with "---"
        firstline = vf.readline()
        headers = dict()
        if yamlmatch.match(firstline):
            line = vf.readline()
            headerstring = ''
            # Read the headers.
            # Go until we get to the terminator to end the headers
            while not yamlmatch.match(line):
                headerstring += line
                line = vf.readline()
                if not line:
                    raise InputError(
                        'Markdown file {} YAML headers don\'t end in a YAML terminator (...,---,===)'.format(file_loc))
            headers = yaml.safe_load(headerstring)  # Retrieve the YAML headers
            if headers == None:
                headers = {}
            markdown = vf.read()  # Read the rest of the file
        elif jsonmatch.match(firstline):
            line = vf.readline()
            headerstring = ''
            # Read the headers.
            # Go until we get to the terminator to end the headers
            while not jsonmatch.match(line):
                headerstring += line
                line = vf.readline()
                if not line:
                    raise InputError(
                        'Markdown file {} JSON headers don\'t end in a JSON terminator (;;;)'.format(file_loc))
            headers = json.loads(headerstring)  # Retrieve the JSON headers
            markdown = vf.read()  # Read the rest of the file
        else:
            # There's no headers in this file. Put the first line back and just add it all into the markdown
            markdown = firstline + '\n' + vf.read()

        # Begin post processing. Process includes and imports.

        # Process includes in headers
        if process_includes and 'include' in headers:
            headers = _process_includes(headers, includemapper)

        # Process imports in headers
        if process_imports and 'import' in headers:
            headers = _process_imports(headers)

        return headers, markdown


def markdown_from_directory(dir_loc):
    log.debug("Reading markdown from directory {}".format(dir_loc))

    if not os.path.isdir(dir_loc):
        raise InputError("dir_loc:{0} is not a valid path".format(dir_loc))

    files = []

    for f in os.listdir(dir_loc):
        mdfile = os.path.join(dir_loc, f)
        if os.path.isfile(mdfile):
            fn, ext = os.path.splitext(mdfile)
            if ext == ".md" or ext == ".rr":
                files.extend([markdown_from_file(mdfile)])

    return files



import os

# This is to fill in the defaults so there's not too many variables
config = {
    "defaulttemplate": os.getenv('RR_TEMPLATE', '/template/volkis-template.yaml'),
    "input_file": os.getenv('RR_INPUT_FILE', "/report/reportbody.md"),
    "output_file": os.getenv('RR_OUTPUT_FILE', '-'),
    "nessusmapper": os.getenv('RR_NESSUSMAPPER', ''),
    "format": os.getenv('RR_FORMAT', ""),
    "verbose": os.getenv('RR_VERBOSE', False),
    # The template mapper. This gives the locations of template files for each template.
    "templatemapper": {
    },
    # Files with additional parameters
    "includes": []
}



from . import vulnerability
from . import config  # Configuration settings within report ranger
from .errors import InputError
from .template import Template
import os
import jinja2
import logging

log = logging.getLogger(__name__)


def main(args):
    # Turn on verbose mode
    if args.verbose:
        logging.basicConfig(
            format='%(levelname)s: %(message)s', level=logging.INFO)
    else:
        logging.basicConfig(format='%(levelname)s: %(message)s',
                            level=logging.WARNING)

    parentdir = os.path.dirname(os.path.join(os.path.curdir, args.input))

    # We need to change the current working directory to the directory of the template otherwise relative
    # paths inside the template won't work. For instance you won't be able to include executivesummary.md
    rr_parent_folder = os.path.abspath(os.path.curdir)

    # Get the absolute location of the template so it survives the change of directory

    if args.template:
        if args.template in config.config['templatemapper']:
            templatefile = config.config['templatemapper'][args.template]
        else:
            templatefile = os.path.abspath(args.template)
    else:
        templatefile = ''

    os.chdir(parentdir)
    parentdir = '.'
    mdfile = os.path.basename(args.input)

    # Get the template
    template = Template(
        templatefile, config.config['templatemapper'], config.config['defaulttemplate'])

    # Get the extension of the output file
    fn, ext = os.path.splitext(args.output)

    # Figure out what target we have
    if args.format == "pdf":
        target = "latex"
        docformat = "pdf"
    elif args.format == "latex":
        target = "latex"
        docformat = "markdown"
    elif args.format == "markdown":
        target = "markdown"
        docformat = "markdown"
    elif args.format == "docx":
        target = "docx"
        docformat = "docx"
    elif args.format == "html":
        target = "html"
        docformat = "html"
    elif args.format == "csv":
        target = "csv"
        docformat = "csv"
    else:
        if ext == ".docx":
            target = "docx"
            docformat = "docx"
            log.info("Setting target and format to docx")
        elif ext == ".md" or ext == ".rr":
            target = "markdown"
            docformat = "md"
            log.info("Setting target to markdown and format to md")
        elif ext == ".html":
            target = "html"
            docformat = "html"
            log.info("Setting target and format to html")
        elif ext == ".csv":
            target = "csv"
            docformat = "csv"
            log.info("Setting target and format to csv")
        else:  # Default to PDF
            target = "latex"
            docformat = "pdf"
            log.info("Setting target to latex and format to pdf")

    # Pandoc does not support PDF output to stdout, so we need to hack it by
    # making a symlink to /dev/stdout and outputting to that file
    stdout_link = None
    if docformat.lower() == 'pdf' and args.output == '-':
        stdout_link = '/tmp/stdout.pdf'
        os.symlink('/dev/stdout', stdout_link)
        args.output = stdout_link

    # Convert output file path into full path if relative path is given
    if args.output[0] != '/':
        args.output = os.path.join(rr_parent_folder, args.output)

    try:
        output = template.process_file(mdfile, target, docformat, args.output)
    except InputError as ie:
        log.error("Input Error: {}".format(ie.message))
        exit()
    except jinja2.exceptions.TemplateSyntaxError as error:
        log.error("Jinja2 error: {} at lineno {} for file {}".format(
            error.message, error.lineno, error.filename))
        exit()

    # If we're outputting to stdout, remove the link
    if stdout_link and os.path.exists(stdout_link):
        os.remove(stdout_link)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['report_ranger']

package_data = \
{'': ['*']}

install_requires = \
['argparse',
 'cerberus',
 'jinja2',
 'kaleido==0.2.1',
 'mistune',
 'num2words',
 'numpy',
 'openpyxl',
 'pandas',
 'plotly',
 'pyyaml',
 'tabulate']

setup_kwargs = {
    'name': 'report-ranger',
    'version': '1.0',
    'description': '',
    'long_description': None,
    'author': 'Matthew Strahan',
    'author_email': 'matt@volkis.com.au',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
